////////////////////////////////////////////////////////////////////////////////
// TJDUrlLabel 1.5
//
// Target: C++ Builder/Delphi 32 bit
//
// Copyright (c) 2000-2002 Jacob "m3Rlin" Dybala. Freeware.
//   WWW  : http://www.m3rlin.prv.pl/, http://www.delphifaq.prv.pl/
//   EMail: <m3rlin@m3rlin.prv.pl>
// Portions (c) 1998 Ray Adams (TRUrlLabel)
//
// Created : March 12 2000
// Modified: April 17 2002
//
// You may use this component in any type of application (public domain,
// freeware, shareware, or even commercial). You do not have to give me credit,
// but I would greatly appreciate such a thing. Please let me know of apps that
// use my component and how you like it. Also please send me any changes you
// make so that I can share them with others. You got the component free so
// why not let others use it?
//
// History
// 1.0 [March 12 2000]
//   Initial version.
// 1.1 [April 3 2000]
//   + Added CreateUrl() and SetUrl() function.
//   + Added ShowUrlHint property.
//   * Default font name changed from MS Sans Serif to Verdana.
//   * Now Tm3rUrlLabel is always underlined if DetectMouse is False.
// 1.2 [April 8 2000]
//   + Added MailBody property.
//   + 6 new URL types added:
//     utFile, utGopher, utHttps, utNews, utTelnet and utWais.
//   * Now Hint property is updated after mail subject change.
//   * Fixed CreatUrl() bug. If no subject was specified it still would leave
//     the '?Subject=' part.
//   * 'ut' prefix added to URL types.
// 1.21 [April 14 2000]
//   + Added Verdana font presence check on create.
// 1.3 [April 18 2000]
//   + Added Cc and Bcc properties.
//   * Added mail body #13#10 sequence replacing.
//   * Fixed UrlHint setting if URL type is set after URL.
//   * Body property is now TStrings instead of string type.
// 1.5 [April 17 2002]
//   + Added LaunchUrl() routine.
//   + Added UnderlineOnEnter property
//   * Tahoma is the default font.
//   * The component is now installed on the 'Internet' tab.
////////////////////////////////////////////////////////////////////////////////

unit JDUrlLabel;

interface

uses
  Classes, Graphics, Controls, Forms, StdCtrls, Messages, Windows;

type
  TUrlType = (
    utCustom, utFile, utFtp, utGopher, utHttp, utHttps, utMail, utNews,
    utTelnet, utWais
  );

  TJDUrlLabel = class(TLabel)
  private
    FDetectMouse,
    FShowUrlHint,
    FUnderlineOnEnter: Boolean;
    FBcc,
    FCc,
    FCustomUrlType,
    FMailSubject,
    FURL             : string;
    FActiveColor,
    FInactiveColor   : TColor;
    FOnMouseEnter,
    FOnMouseLeave    : TNotifyEvent;
    FMailBody        : TStrings;
    FUrlType         : TUrlType;
    function CreateUrl: string;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
  protected
    procedure SetBcc(Value: string);
    procedure SetCc(Value: string);
    procedure SetDetectMouse(Value: Boolean);
    procedure SetMailBody(Value: TStrings);
    procedure SetMailSubject(Value: string);
    procedure SetUrl(Value: string);
    procedure SetUrlHint(Value: Boolean);
    procedure SetUrlType(Value: TUrlType);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
    procedure LaunchUrl(sURL: string);
  published
    { Properties }
    property ActiveColor: TColor read FActiveColor write FActiveColor
      default clBlue;
    property Bcc: string read FBcc write SetBcc;
    property Cc: string read FCc write SetCc;
    property CustomURLType: string read FCustomUrlType write FCustomUrlType;
    property DetectMouse: Boolean read FDetectMouse write SetDetectMouse;
    property InactiveColor: TColor read FInactiveColor write FInactiveColor
      default clNavy;
    property MailBody: TStrings read FMailBody write SetMailBody;
    property MailSubject: string read FMailSubject write SetMailSubject;
    property ShowUrlHint: Boolean read FShowUrlHint write SetUrlHint
      default True;
    property UnderlineOnEnter: Boolean read FUnderlineOnEnter
      write FUnderlineOnEnter default True; 
    property URLType: TUrlType read FUrlType write SetUrlType default utHttp;
    property URL: string read FURL write SetURL;
    { Events }
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

procedure Register;

implementation

uses
  ShellApi, SysUtils;

{ Tm3rUrlLabel }

constructor TJDUrlLabel.Create(AOwner: TComponent);
begin
  inherited;
  with Font do begin
    Color := clNavy;
    if Screen.Fonts.IndexOf('Tahoma') <> -1 then
      Name := 'Tahoma'
    else
      Name := 'Arial';
  end;
  Cursor := crHandPoint;
  FMailBody := TStringList.Create;
  FDetectMouse      := True;
  FUnderlineOnEnter := True;
  ActiveColor   := clBlue;
  InactiveColor := clNavy;
  ShowUrlHint   := True;
end;

destructor TJDUrlLabel.Destroy;
begin
  FMailBody.Free;
  inherited;
end;

procedure TJDUrlLabel.CMMouseEnter(var Msg: TMessage);
begin
  if FDetectMouse then begin
    Font.Color := FActiveColor;
    if FUnderlineOnEnter then
      Font.Style := [fsUnderline];
  end;
  inherited;
  if Assigned(FOnMouseEnter) then
    fOnMouseEnter(Self);
end;

procedure TJDUrlLabel.CMMouseLeave(var Msg: TMessage);
begin
  if FDetectMouse then begin
    Font.Color := FInActiveColor;
    Font.Style := [];
  end;
  inherited;
  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(Self);
end;

procedure TJDUrlLabel.SetBcc(Value: string);
begin
  FBcc := Value;
  if FShowUrlHint then
    Hint := CreateUrl;
end;

procedure TJDUrlLabel.SetCc(Value: string);
begin
  FCc := Value;
  if FShowUrlHint then
    Hint := CreateUrl;
end;

procedure TJDUrlLabel.SetDetectMouse(Value: Boolean);
begin
  FDetectMouse := Value;
  if Value then
    Font.Style := Font.Style - [fsUnderline]
  else
    Font.Style := Font.Style + [fsUnderline];
end;

procedure TJDUrlLabel.SetMailBody(Value: TStrings);
begin
  FMailBody.Assign(Value);
  if FShowUrlHint then
    Hint := CreateUrl;
end;

procedure TJDUrlLabel.SetMailSubject(Value: string);
begin
  FMailSubject := Value;
  if FShowUrlHint then
    Hint := CreateUrl;
end;

procedure TJDUrlLabel.SetUrl(Value: string);
begin
  FUrl := Value;
  if FShowUrlHint then
    Hint := CreateUrl;
end;

procedure TJDUrlLabel.SetUrlHint(Value: Boolean);
begin
  FShowUrlHint := Value;
  if Value then
    Hint := CreateUrl
  else if Hint = CreateUrl then
    Hint := '';
end;

procedure TJDUrlLabel.SetUrlType(Value: TUrlType);
begin
  FUrlType := Value;
  if FShowUrlHint then
    Hint := CreateUrl;
end;

function TJDUrlLabel.CreateUrl: string;

  function Replace(const sSubString, sNewSubString, sString: string): string;
  var
    iI   : Integer;
    sTemp: string;
  begin
    sTemp := sString;
    repeat
      iI := Pos(LowerCase(sSubString), LowerCase(sTemp));
      if iI > 0 then begin
        Result := Result + Copy(sTemp, 1, iI - 1) + sNewSubString;
        sTemp := Copy(sTemp, iI + Length(sSubString), MaxInt);
      end else
        Result := Result + sTemp;
    until iI <= 0;
  end;

begin
  case FUrlType of
    utCustom: Result := FCustomUrlType + FURL;
    utFile  : Result := 'file://' + FURL;
    utFtp   : Result := 'ftp://' + FURL;
    utGopher: Result := 'gopher://' + FURL;
    utHttp  : Result := 'http://' + FURL;
    utHttps : Result := 'https://' + FURL;
    utMail  : begin
      Result := 'mailto:' + FURL;
      if fMailSubject <> '' then
        Result := Result + '?Subject=' + FMailSubject;
      if fMailBody.Text <> '' then
        Result := Result + '&Body=' + Replace(#13#10, '%0A%0D', FMailBody.Text);
      if fBcc <> '' then
        Result := Result + '&Bcc=' + FBcc;
      if fCc <> '' then
        Result := Result + '&Cc=' + FCc;
    end;
    utNews  : Result := 'news:' + FURL;
    utTelnet: Result := 'telnet:' + FURL;
    utWais  : Result := 'wais:' + FURL;
  end;
end;

procedure TJDUrlLabel.Click;
begin
  LaunchUrl(CreateUrl);
end;

procedure TJDUrlLabel.LaunchUrl(sURL: string);
begin
  ShellExecute(GetDesktopWindow(), 'open', PChar(sURL), nil, nil, SW_SHOWNORMAL);
end;

procedure Register;
begin
  RegisterComponents('Internet', [TJDUrlLabel]);
end;

end.
